#!/usr/bin/env python
#
# Licensed under the GNU GPL.

import gobject

import gtk, gnomeapplet
import dbus
import dbus.glib

import getopt, sys
from os.path import *

# Allow to use uninstalled
def _check(path):
	return exists(path) and isdir(path) and isfile(path+"/AUTHORS")

name = join(dirname(__file__), '..')
if _check(name):
	print 'Running uninstalled cohoba, modifying PYTHONPATH'
	sys.path.insert(0, abspath(name))
else:
	sys.path.insert(0, abspath("@PYTHONDIR@"))
	print "Running installed cohoba, using [@PYTHONDIR@:$PYTHONPATH]"

# Now the path is set, import our applet
import cohoba, cohoba.defs

try:
	# attempt to set a name for killall
	import cohoba.osutils
	cohoba.osutils.set_process_name ("cohoba")
except:
	print "Unable to set processName"

import gettext, locale
gettext.bindtextdomain('cohoba', abspath(join(cohoba.defs.DATA_DIR, "locale")))
if hasattr(gettext, 'bind_textdomain_codeset'):
	gettext.bind_textdomain_codeset('cohoba','UTF-8')
gettext.textdomain('cohoba')

locale.bindtextdomain('cohoba', abspath(join(cohoba.defs.DATA_DIR, "locale")))
if hasattr(locale, 'bind_textdomain_codeset'):
	locale.bind_textdomain_codeset('cohoba','UTF-8')
locale.textdomain('cohoba')

def applet_factory(applet, iid):
	print 'Starting Cohoba instance:', applet, iid
	from cohoba.CohobaApplet import CohobaApplet
	CohobaApplet(applet)
	return True

# Return a standalone window that holds the applet
def build_window():
	app = gtk.Window(gtk.WINDOW_TOPLEVEL)
	app.set_title("Cohoba")
	app.connect("destroy", gtk.main_quit)
	app.set_property('resizable', False)
	
	applet = gnomeapplet.Applet()
	applet.get_orient = lambda: gnomeapplet.ORIENT_DOWN
	applet_factory(applet, None)
	applet.reparent(app)
		
	app.show_all()
	
	return app
		
		
def usage():
	print """=== Cohoba: Usage
$ cohoba [OPTIONS]

OPTIONS:
	-h, --help			Print this help notice.
	-w, --window		Launch the applet in a standalone window for test purposes (default=no).
	"""
	sys.exit()
	
if __name__ == "__main__":	
	standalone = False
	test = False
	
	try:
		opts, args = getopt.getopt(sys.argv[1:], "hwt", ["help", "window", "test"])
	except getopt.GetoptError:
		# Unknown args were passed, we fallback to bahave as if
		# no options were passed
		opts = []
		args = sys.argv[1:]
	
	args.insert(0, sys.argv[0])
	sys.argv = args
	
	for o, a in opts:
		if o in ("-h", "--help"):
			usage()
		elif o in ("-w", "--window"):
			standalone = True
		elif o in ("-t", "--test"):
			test = True
	
	if test:
		import cohobatest
		cohobatest.run_test()
	else:
		if standalone:
			import gnome
			gnome.init(cohoba.defs.PACKAGE, cohoba.defs.VERSION)
			build_window()
			gtk.main()
		else:
			gnomeapplet.bonobo_factory(
				"OAFIID:Cohoba_Factory",
				gnomeapplet.Applet.__gtype__,
				cohoba.defs.PACKAGE,
				cohoba.defs.VERSION,
				applet_factory)
