import gobject, gtk

from cohoba.telepathy.Constants import CONNECTION_STATUS_DISCONNECTED, CONNECTION_STATUS_REASON_NONE_SPECIFIED, CONNECTION_STATUS_CONNECTED
from cohoba.Utils import global_error_handler, conn_reason_to_string, conn_status_to_string
from cohoba.telepathy.ProtocolRegistry import get_protocol_registry
from cohoba.telepathy.DBusProxyProvider import get_proxy_provider
from cohoba.ui.AccountEditor import get_editor_for_account

class Account(gobject.GObject):
	__gsignals__ = {
		"changed": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
	}
	
	def __init__(self, protocol=None, manager=None):
		gobject.GObject.__init__(self)
	
		self.protocol = protocol
		self.manager = manager
		self.account = None
		self.password = None
		
		self.is_connected = False
		self.auto_connect = False
		self.save_pass = False
		self.manager_iface = None
		self.conn_iface = None
		self.conn_obj_path = None
		
		self.status = conn_status_to_string(CONNECTION_STATUS_DISCONNECTED)
		self.reason = conn_reason_to_string(CONNECTION_STATUS_REASON_NONE_SPECIFIED)
	
	def is_connectable(self):
		return (self.protocol != None and self.manager != None and self.account != None and self.password != None)
		
	def connect_account(self):
		self.connect_manager()
		self.connect_protocol()
				
	def connect_manager(self):
		service = get_protocol_registry().GetBusName(self.manager)
		obj = get_protocol_registry().GetObjectPath(self.manager)
		get_proxy_provider().create_proxy(service, obj)
		self.manager_iface = get_proxy_provider().get_iface(obj, "org.freedesktop.Telepathy.ConnectionManager")

	def connect_protocol(self):
		print '-- Connecting protocol', self.protocol, self.manager
		if self.password == None:
			get_editor_for_account(self).present_with_time(gtk.get_current_event_time())
			return
		
		params = {
			"account":  self.account,
			"password": self.password,
		}
		if self.account.endswith("@gmail.com"):
			params["server"] = "talk.google.com"
			params["old-ssl"] = True
			params["ignore-ssl-errors"] = True
			import dbus
			params["port"] = dbus.UInt32(5223)
			
		self.manager_iface.Connect(self.protocol, params,
			reply_handler=self.on_protocol_connected,
			error_handler=global_error_handler)
		
	def on_protocol_connected(self, service, obj):
		print '-- Manager connected'
		get_proxy_provider().create_proxy(service, obj)
		self.conn_iface = get_proxy_provider().get_iface(obj, "org.freedesktop.Telepathy.Connection")
		self.conn_obj_path = obj
		
		# Retrieve connection status, and wait until connected
		self.conn_iface.connect_to_signal('StatusChanged', self.on_status_changed)
		self.conn_iface.GetStatus(reply_handler=self.on_status_changed, error_handler=global_error_handler)
			
	def on_status_changed(self, status, reason=None):
		print '-- Status change', status, reason
		self.status = conn_status_to_string(status)
		self.reason = conn_reason_to_string(reason)

		if status == CONNECTION_STATUS_CONNECTED:
			self.is_connected = True
		else:
			self.is_connected = False
			
		self.emit('changed')
	
	def disconnect_account(self):
		if self.conn_iface != None:
			print 'Disconnecting:', self.conn_obj_path
			channels = self.conn_iface.ListChannels()
			for path, iface, type, handle in channels:
				iface = get_proxy_provider().get_iface(path, "org.freedesktop.Telepathy.Channel")
				if iface != None:
					try:
						iface.Close()
					except Exception, e:
						print 'Closing %s yield %s' % (path, e)
			self.conn_iface.Disconnect(reply_handler=lambda: None, error_handler=lambda e: global_error_handler(e, "Disconnect:86:Account.py"))
			
		self.manager_iface = None
		self.conn_iface = None
		self.is_connected = False
		
		self.emit('changed')
	
	def changed(self):
		self.emit('changed')
