import gtk, gtk.glade, gobject
from os.path import join
import cohoba
from gettext import gettext as _

from cohoba.ui.AccountEditor import AccountEditor, get_editor_for_account
from cohoba.telepathy.AccountList import get_account_list
from cohoba.ui.ProtocolIcons import get_protocol_icons

class AccountsListView(gtk.TreeView):
	TOGGLE_HEADER = "Auto Connect"
	COL_ICON = 0
	COL_TOGGLE = 1
	COL_TEXT = 2
	
	def __init__(self):
		gtk.TreeView.__init__(self)		

		props = {
			'headers-visible': False,
			"enable-search": False,
			"rules-hint": True,
		}
		for prop, val in props.items():
			self.set_property(prop, val)
			
		self.set_model(get_account_list())
		
		# -- Columns setup
		cell = gtk.CellRendererPixbuf()
		column = gtk.TreeViewColumn("Icon", cell)
		column.set_cell_data_func(cell, self.get_cell_data, self.COL_ICON)
		self.append_column(column)

		cell = gtk.CellRendererToggle()
		cell.connect('toggled', self.on_account_toggled)
		column = gtk.TreeViewColumn(self.TOGGLE_HEADER, cell)
		column.set_cell_data_func(cell, self.get_cell_data, self.COL_TOGGLE)
		self.append_column(column)
				
		cell = gtk.CellRendererText()
		column = gtk.TreeViewColumn("Account", cell)
		column.set_cell_data_func(cell, self.get_cell_data, self.COL_TEXT)
		self.append_column(column)
		# ---
		
		self.connect('destroy', self.on_close)
		self.connect('row-activated', self.on_row_activated)
				
	def get_cell_data(self, column, cell, model, iter, col):
		account = model[iter][0]
		if col == self.COL_ICON:
			cell.set_property("pixbuf", get_protocol_icons().get_icon(account.protocol, account.is_connected))
		elif col == self.COL_TOGGLE:
			cell.set_property("active", account.auto_connect)
		elif col == self.COL_TEXT:
			if account.auto_connect:
				autoconnect_string = "This account connects on startup"
			else:
				autoconnect_string = "This account stays disconnected on startup"
			
			if not account.is_connectable():
				cell.set_property("markup", "<b>%s</b>" % _("New Account"))
			else:
				cell.set_property("markup", "<b>%s</b>\n<span size='small'>%s</span>\n<span size='small'>Currently %s <span foreground='gray'>(%s)</span></span>" % (account.account, autoconnect_string, account.status, account.reason))
			
	def show_account_editor(self, account):
		editor = get_editor_for_account(account)
		editor.present_with_time(gtk.get_current_event_time())

	def show_selected_account_editor(self):
		model, iter = self.get_selection().get_selected()
		if iter == None:
			return
		
		self.show_account_editor(self.get_model()[iter][0])
				
	def remove_selected(self):
		model, iter = self.get_selection().get_selected()
		if iter == None:
			return

		get_account_list().remove_account(model[iter][0])
	
	def connect_selected(self, connect):
		model, iter = self.get_selection().get_selected()
		if iter == None:
			return
			
		if connect:
			self.get_model()[iter][0].connect_account()
		else:
			self.get_model()[iter][0].disconnect_account()
			
	def on_row_activated(self, treeview, path, view_column):
		self.show_selected_account_editor()
	
	def on_close(self, win):
		get_account_list().save()
		
	def on_account_toggled(self, cell, path):
		account = self.get_model()[path][0]
		account.auto_connect = (not cell.get_active())
		account.changed()

class AccountsConnectListView(AccountsListView):
	TOGGLE_HEADER = "Connected"
	
	def __init__(self):
		AccountsListView.__init__(self)
	
	def get_cell_data(self, column, cell, model, iter, col):
		account = model[iter][0]
		
		if col == self.COL_ICON:
			cell.set_property("pixbuf", get_protocol_icons().get_icon(account.protocol, account.is_connected))
		elif col == self.COL_TOGGLE:
			cell.set_property("active", account.is_connected)
		elif col == self.COL_TEXT:
			cell.set_property("markup", "<b>%s</b>\n<span size='small'>%s <span foreground='gray'>(%s)</span></span>" % (account.account, account.status, account.reason))
			
	def on_account_toggled(self, cell, path):
		if not cell.get_active():
			self.get_model()[path][0].connect_account()
		else:
			self.get_model()[path][0].disconnect_account()

