import gtk, gobject, cohoba, pango
from os.path import join

from cohoba.telepathy.DBusProxyProvider import get_proxy_provider
from cohoba.telepathy.Constants import CONNECTION_HANDLE_TYPE_CONTACT
from cohoba.telepathy.MergedContactList import get_contact_list, MergedContactList
from cohoba.telepathy.Contact import Contact
from cohoba.telepathy.ContactGroup import ContactGroup, get_group_filter
from cohoba.ui.AccountComboBox import AccountComboBox
from cohoba.ui.ContactListWindowManager import get_contact_list_window_manager
from cohoba.telepathy.ChannelHandler import get_channel_handler

# Inheriting CellLayout is a workaround for a bug fixed in 2.8.2 < PyGtk <=
# 2.8.6.

class ContactListView(gtk.IconView, gtk.CellLayout):
	def __init__(self, group=None):
		if group == None:
			self.model = get_contact_list()
		else:
			self.model = get_group_filter(group, get_contact_list())
			
		gtk.IconView.__init__(self, self.model.filter_new())
		
		pixbuf = gtk.CellRendererPixbuf()
		self.pack_start(pixbuf)
		self.set_attributes(pixbuf, pixbuf=MergedContactList.COL_PIXBUF)
		
		markup = gtk.CellRendererText ()
		markup.set_property("ellipsize", pango.ELLIPSIZE_END)
		markup.set_property("width-chars", 40)
		self.pack_start (markup)
		self.set_attributes (markup, markup=MergedContactList.COL_STRING)
		
		self.set_property("reorderable", True)
		self.set_property("selection-mode", gtk.SELECTION_MULTIPLE)
		self.set_property("orientation", gtk.ORIENTATION_HORIZONTAL)
		
		self.get_model().set_visible_func(self.on_filter_row)
		
		self.connect('item-activated', self.on_contact_activated)
		self.connect('button-press-event', self.on_button_press)
			
	def on_button_press(self, view, event):
		if event.button != 3:
			return
			
		path = self.get_path_at_pos(int(event.x), int(event.y))
		menu = gtk.Menu()
		
		if path == None:
			item = gtk.MenuItem("Add...")
			item.connect("activate", self.on_add_contact)
			menu.append(item)
			
			item = gtk.MenuItem("Add Group...")
			item.connect("activate", self.on_add_contact_group)
			menu.append(item)
		else:
			contact = self.get_model()[path][MergedContactList.COL_CONTACT]
			if contact.__class__ == Contact:
				item = gtk.MenuItem("Chat")
				item.connect("activate", lambda item: self.on_contact_activated(view, path))
				menu.append(item)
				
				item = gtk.MenuItem("Remove")
				item.connect("activate", self.on_remove_contact, contact)
				menu.append(item)
				
				item = gtk.MenuItem("Groups...")
				item.connect("activate", self.on_contact_groups, contact)
				menu.append(item)
			
			if contact.__class__ == ContactGroup:
				item = gtk.MenuItem("Remove")
				item.connect("activate", self.on_remove_contact_group, path)
				menu.append(item)
				
		menu.show_all()
		menu.popup(None, None, None, event.button, event.time)
	
	def on_remove_contact(self, item, contact):
		self.model.remove_member(contact.conn_path, contact.handle)
	
	def on_add_contact(self, item):
		glade = gtk.glade.XML(join(cohoba.SHARED_DATA_DIR, "main.glade"), "add_contact_dialog")
		dialog = glade.get_widget("add_contact_dialog")
		entry = glade.get_widget("name")
		
		accounts = AccountComboBox()
		accounts.set_active(0)
		accounts.show()
		
		glade.get_widget("account_table").attach(accounts, 1, 2, 0, 1, xoptions=gtk.FILL, yoptions=gtk.FILL)
		
		response = dialog.run()
		dialog.destroy()
		if response == gtk.RESPONSE_OK and entry.get_text() != "":
			account = accounts.get_selected_account()
			self.model.add_member(account.conn_obj_path, entry.get_text())
	
	def on_contact_groups(self, item, contact):
		glade = gtk.glade.XML(join(cohoba.SHARED_DATA_DIR, "main.glade"), "contact_groups_dialog")
		dialog = glade.get_widget("contact_groups_dialog")
		entry = glade.get_widget("groups")
		entry.set_text(' '.join(contact.groups))
		
		entry2 = glade.get_widget("name")
		entry2.set_text(str(contact.favorite))
		
		response = dialog.run()
		dialog.destroy()
		if response == gtk.RESPONSE_OK:
			contact.groups = [group for group in entry.get_text().split(" ") if group.strip() != ""]
			contact.favorite = (entry2.get_text() != "")
			contact.emit('updated')
	
	def on_add_contact_group(self, item):
		glade = gtk.glade.XML(join(cohoba.SHARED_DATA_DIR, "main.glade"), "contact_groups_dialog")
		dialog = glade.get_widget("contact_groups_dialog")
		groups_entry = glade.get_widget("groups")
		name_entry = glade.get_widget("name")
		
		response = dialog.run()
		dialog.destroy()
		if response == gtk.RESPONSE_OK and name_entry.get_text() != "":
			groups = [group for group in groups_entry.get_text().split(" ") if group.strip() != ""]
			self.model.add_contact_group(name_entry.get_text(), groups)
		
	def on_remove_contact_group(self, item, path):
		self.model.remove(self.model.get_iter(path))
		
	def show_contact_group(self, group):
		w=gtk.Window()
		w.add(ContactListView(group))
		w.show_all()
		
	def on_filter_row(self, model, iter):
		contact = model[iter][MergedContactList.COL_CONTACT]
		#return (contact != None and contact.status_code != 'offline')
		return True
		
	def on_contact_activated(self, view, path):
		contact = self.get_model()[path][MergedContactList.COL_CONTACT]
		if contact.__class__ == Contact:
			get_channel_handler().start_conversation(contact)
		else:
			get_contact_list_window_manager().show(contact)
